ContainerWidget {
  layout: horizontal;
  background: $boost;
  height: 5;
  margin: 1;
  min-width: 50;
  padding: 1;
}

.container-info {
  width: 40;
}

.status-text {
  width: 12;
  text-align: center;
}

StatusButtons Button {
  margin: 0 2;
}

.running {
  background: $error;
}

.exited {
  background: $success;
}
